from __future__ import annotations

import os
import tomllib
from pathlib import Path
from dataclasses import dataclass


@dataclass(slots=True)
class ApibeanOptions:
    base_url: str = "http://localhost:8000"
    login_path: str = "/auth/login"
    username: str = "admin"
    password: str = "admin"
    timeout: float = 10.0
    auto_login: bool = True


_DEFAULTS = ApibeanOptions()


def _load_pyproject_options() -> dict:
    """
    Load [tool.pytest.apibean.options] từ pyproject.toml
    """
    for path in (Path.cwd(), *Path.cwd().parents):
        pyproject = path / "pyproject.toml"
        if pyproject.exists():
            with pyproject.open("rb") as f:
                data = tomllib.load(f)
            return (
                data
                .get("tool", {})
                .get("pytest", {})
                .get("apibean", {})
                .get("options", {})
            )
    return {}


def load_settings() -> ApibeanOptions:
    defaults = _DEFAULTS
    opts = _load_pyproject_options()

    return ApibeanOptions(
        base_url=opts.get(
            "base_url",
            os.getenv("API_BASE_URL", defaults.base_url),
        ),
        login_path=opts.get(
            "login_path",
            os.getenv("API_LOGIN_PATH", defaults.login_path),
        ),
        username=opts.get(
            "username",
            os.getenv("API_USERNAME", defaults.username),
        ),
        password=opts.get(
            "password",
            os.getenv("API_PASSWORD", defaults.password),
        ),
        timeout=float(
            opts.get(
                "timeout",
                os.getenv("API_TIMEOUT", defaults.timeout),
            )
        ),
        auto_login=bool(opts.get("auto_login", defaults.auto_login)),
    )


settings = load_settings()



from dataclasses import asdict

import pytest

from .fixtures.auth import *


def pytest_addoption(parser):
    parser.addoption(
        "--apibean-show-config",
        action="store_true",
        help="Show apibean pytest configuration",
    )


def pytest_cmdline_main(config):
    if config.getoption("--apibean-show-config"):
        from .settings import settings
        print("\n[pytest-apibean config]")
        for k, v in asdict(settings).items():
            print(f"  {k} = {v}")
    return pytest.ExitCode.OK



def test_plugin_loaded(pytestconfig):
    plugin = pytestconfig.pluginmanager.get_plugin("pytest_apibean")
    assert plugin is not None

def test_plugin_distinfo(pytestconfig):
    plugins = pytestconfig.pluginmanager.list_plugin_distinfo()
    plugin_modules = [dist for (dist, _) in plugins]
    import apibean.pytest
    assert apibean.pytest.plugin in plugin_modules
